local dialog = nil
local vb = nil

class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
function RenoiseScriptingTool:__init()    
  renoise.Document.DocumentNode.__init(self) 
  self:add_property("Name", "Untitled Tool")
  self:add_property("Version", "1.0")
end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_version = manifest:property("Version").value

local function transpose(shift)
  local dropped_notes = 0
  local rs = renoise.song()
  local pattern = rs.selected_pattern
  local track = rs.selected_track
  local pattern_track = pattern.tracks[rs.selected_track_index]
  local note_column = nil
  -- check to see if any notes will be dropped
  for _, line in ipairs(pattern_track.lines) do
    if (not line.is_empty) then
      for note_column_index = 1, track.visible_note_columns, 1 do
        note_column = line.note_columns[note_column_index]
        if (not note_column.is_empty) then
          if (note_column.note_value < 120) then
            if (((note_column.note_value + shift) < 0) or ((note_column.note_value + shift) > 119)) then
              dropped_notes = dropped_notes + 1
            end
          end          
        end
      end    
    end
  end
  
  -- if any notes would be dropped, then cancel and do nothing
  if (dropped_notes > 0) then
    return
  else
    -- else, transpose the notes
    for _, line in ipairs(pattern_track.lines) do
      if (not line.is_empty) then
        for note_column_index = 1, track.visible_note_columns, 1 do
          note_column = line.note_columns[note_column_index]
          if (not note_column.is_empty) then
            if (note_column.note_value < 120) then
              note_column.note_value = math.min(119, math.max(0, note_column.note_value + shift))
            end
          end
        end    
      end
    end
  end
  
end

local function reset(base_note)
  local rs = renoise.song()
  local pattern = rs.selected_pattern
  local track = rs.selected_track
  local pattern_track = pattern.tracks[rs.selected_track_index]
  local note_column = nil
  
  -- find the first note in the track
  local found_note = -1
  for _, line in ipairs(pattern_track.lines) do
    if (not line.is_empty) then
      for note_column_index = 1, track.visible_note_columns, 1 do
        note_column = line.note_columns[note_column_index]
        if (not note_column.is_empty) then
          if (note_column.note_value < 120) then
            found_note = note_column.note_value
            break    
          end      
        end
      end    
    end
    if (found_note > -1) then
      break
    end
  end
  
  if (found_note > -1) then
    local shift = base_note - found_note
    transpose(shift)
  end
  
end

-- Add keybindings to reset base note
for i = 0, 9, 1 do
  renoise.tool():add_keybinding {
    name = "Pattern Editor:Track Operations:Reset to C-" .. tostring(i),
    invoke = function() reset(i * 12) end
  }
end
  
-- Add keybindings for transposing up
for i = 1, 12, 1 do
  renoise.tool():add_keybinding {
    name = "Pattern Editor:Track Operations:Transpose +" .. tostring(i),
    invoke = function() transpose(i) end
  }
end

-- Add keybindings for transposing down
for i = 1, 12, 1 do
  renoise.tool():add_keybinding {
    name = "Pattern Editor:Track Operations:Transpose -" .. tostring(-i),
    invoke = function() transpose(-i) end
  }
end
